use tokio::sync::mpsc;
use tokio::sync::mpsc::error::TryRecvError;

use crate::message::Message;

struct Player {
    name: String,
    channel: Option<Channel>,
    seat: Option<usize>,
}

#[derive(Default)]
pub struct GameController {
    players: Vec<Player>,
}

pub struct Channel {
    pub tx: mpsc::Sender<Message>,
    pub rx: mpsc::Receiver<Message>,
}

impl GameController {
    pub fn try_recv(&mut self) -> Result<Message, TryRecvError> {
        for player in self.players.iter_mut() {
            if let Some(channel) = &mut player.channel {
                let res = channel.rx.try_recv();
                if !(res == Err(TryRecvError::Empty)) {
                    return res
                }
            }
        }
        Err(TryRecvError::Empty)
    }
    pub async fn broadcast(&mut self, msg: Message) {
        for player in self.players.iter() {
            if let Some(channel) = &player.channel {
                channel.tx.send(msg.clone()).await.unwrap();
            }
        }
    }
    pub fn push(&mut self, channel: Channel) {
        let player = Player {
            name: "Guest".to_string(),
            channel: Some(channel),
            seat: None, // TODO
        };
        self.players.push(player);
    }
}

pub fn channel_pair() -> (Channel, Channel) {
    let (atx, brx) = mpsc::channel(32);
    let (btx, arx) = mpsc::channel(32);
    (Channel { tx: atx, rx: arx }, Channel { tx: btx, rx: brx })
}

impl GameController {
    pub async fn poll(&mut self) {
        if let Ok(msg) = self.try_recv() {
            self.dispatch(msg).await;
        }
    }
    async fn dispatch(&mut self, msg: Message) {
        match msg.command.as_str() {
            "CHAT" => self.broadcast(msg).await,
            _ => ()
        };
    }
}
